#!/usr/bin/python
# -*- coding: UTF-8 -*-

from __future__ import division

import os
import time
import shutil
import socket

from PIL import Image
from flask import request, abort

from MDmisc.elist import ifany
from NIST.fingerprint import NISTf_auto

fpcfield = {
    4: 4,
    14: 13,
    15: 13,
}

def get_ip():
    if 'X-Forwarded-For' in request.headers:
        return 'forwarded', request.headers[ 'X-Forwarded-For' ]
    elif 'X-Real-IP' in request.headers:
        return 'forwarded', request.headers[ 'X-Real-IP' ]
    else:
        return 'direct', request.remote_addr

def checktime( UPLOAD_FOLDER, iphash, uuid ):
    if os.path.isfile( UPLOAD_FOLDER + "/" + iphash + "/" + uuid + "/uploadtime.txt" ):
        with open( UPLOAD_FOLDER + "/" + iphash + "/" + uuid + "/uploadtime.txt", "r" ) as fp:
            d = int( fp.read() )
            if time.time() - d > 300:
                abort( 404 )

def process( infile, wd, filename ):
    n = NISTf_auto( infile )
    h = n.hash()
    
    try:
        os.makedirs( wd + "/" + h )
    except:
        pass
    
    if os.path.isfile( wd + "/" + h + "/done.txt" ):
        try:
            os.remove( wd + "/" + h + "/done.txt" )
        except:
            pass
    
    shutil.move( infile, wd + "/" + h + "/" + filename )
    
    ############################################################################
    #    Add the file with the upload timestamp
    
    with open( os.path.join( wd, h, "uploadtime.txt" ), "w+" ) as fp:
        fp.write( "%d" % time.time() )
    
    ############################################################################
    #    Data export
    
    with open( wd + "/" + h + "/data.txt", "w+" ) as fp:
        fp.write( str( n ) )
    
    ############################################################################
    #    Latent export
    
    if 13 in n.get_ntype():
        for idc in n.get_idc( 13 ):
            res = n.get_resolution( idc = idc )
            
            if not os.path.isfile( wd + "/" + h + "/Latent_" + str( idc ) + ".tiff" ):
                n.get_latent( format = "PIL" ).save( 
                    wd + "/" + h + "/Latent_" + str( idc ) + ".tiff",
                    dpi = ( res, res )
                )
            
            n.export_latent_annotated( 
                wd + "/" + h + "/Latent_annotated_" + str( idc ) + ".tiff"
            )
    
    ############################################################################
    #    Fingerprint export
    
    for ntype in [ 4, 14 ]:
        if ntype in n.get_ntype():
            for idc in n.get_idc( ntype ):
                try:
                    fpc = n.get_field( ( ntype, fpcfield[ ntype ] ), idc )
                    
                    if not os.path.isfile( wd + "/" + h + "/F" + str( fpc ) + ".tiff" ):
                        res = n.get_resolution( idc = idc )
                        
                        tmp = n.get_print( format = "PIL", idc = idc )
                        tmp.save( 
                            wd + "/" + h + "/F" + str( fpc ) + ".tiff",
                            dpi = ( res, res )
                        )
                        
                        width, height = n.get_size( idc )
                        if fpc <= 10:
                            targetsize = 150
                        else:
                            targetsize = 250
                        
                        fac = targetsize / width
                        tmp = tmp.resize( ( int( width * fac ), int( height * fac ) ), Image.BICUBIC )
                        tmp.save( 
                            wd + "/" + h + "/F" + str( fpc ) + "_preview.png"
                        )
                
                except:
                    pass
    
    ############################################################################
    #    Tenprint cards export
    
    outres = 1000
    if ifany( [ 4, 14 ], n.get_ntype() ):
        if not os.path.isfile( wd + "/" + h + "/tenprint_card_front.tiff" ):
            n.get_tenprintcard_front( outres ).save( 
                wd + "/" + h + "/tenprint_card_front.tiff",
                dpi = ( outres, outres )
            )
    
    if 15 in n.get_ntype():
        for fpc in [ 22, 24, 25, 27 ]:
            try:
                if not os.path.isfile( wd + "/" + h + "/" + str( fpc ) + ".tiff" ):
                    p = n.get_palmar( "PIL", fpc = fpc )
                    
                    try:
                        res, _ = p.info[ 'dpi' ]
                    except:
                        res = n.get_resolution( n.get_idc_for_fpc( 15, fpc ) )
                    
                    width, height = p.size
                    fac = outres / res
                    if fac != 1:
                        p = p.resize( ( int( width * fac ), int( height * fac ) ), Image.BICUBIC )
                    
                    p.save( 
                        wd + "/" + h + "/" + str( fpc ) + ".tiff",
                        dpi = ( outres, outres )
                    )
            
            except:
                continue
        
        if not os.path.isfile( wd + "/" + h + "/tenprint_card_back.tiff" ):
            n.get_tenprintcard_back( outres ).save( 
                wd + "/" + h + "/tenprint_card_back.tiff",
                dpi = ( outres, outres )
            )
    
    ############################################################################
    #    Write to the end file
    
    with open( wd + "/" + h + "/done.txt", "w+" ) as fp:
        fp.write( "done" )
